/* 
**
** file name: logfunc.h
**
** Header file include by logfunc.c
**
*/

#ifndef LOGFUNC__H
#define LOGFUNC__H

#define LogFlag(fp) \
    LogMsg(fp, "%s: %s: %d", FILE_MACRO, FUNCTION_MACRO, LINE_MACRO)

void *
OpenLog (const char *prog_name, const char *ident, 
		int option, int priority);

void 
LogRet(void *fp, const char *fmt, ...);

void 
LogSys(void *fp, const char *fmt, ...);

void 
LogMsg(void *fp, const char *fmt, ...);

void 
LogQuit(void *fp, const char *fmt, ...);

void 
GetErrnoMsg(char *sp, const char *fmt, ...);

void
CloseLog(void *fp);

#endif
